;This program is free software: you can redistribute it and/or modify
;it under the terms of the Attribution-NonCommercial 4.0 International
;as published by Creative Commons.
 
 ;This program is distributed in the hope that it will be useful,
 ;but WITHOUT ANY WARRANTY; without even the implied warranty of
 ;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 ;
 ;You should have received a copy of the  Attribution-NonCommercial 4.0 International as published by
 ;Creative Commons or visit http://creativecommons.org/licenses/by-nc/4.0/

; Original Author: Jeffrey Nygaard
; Company: Northern Lights Electronic Design, LLC
; Contact: JNygaard@NLEDShop.com
; Date Updated: April 1, 2014 
; Webpage: www.NLEDShop.com/extdisplay

	LIST      P=PIC16F1824          ; list directive to define processor
	#INCLUDE <P16F1824.INC>         ; processor specific variable definitions

	__CONFIG _CONFIG1, _FOSC_INTOSC & _WDTE_OFF & _PWRTE_ON & _MCLRE_OFF & _CP_ON & _CPD_OFF & _BOREN_OFF & _CLKOUTEN_OFF & _IESO_OFF & _FCMEN_OFF
	__CONFIG _CONFIG2, _WRT_OFF & _PLLEN_ON & _STVREN_OFF & _BORV_19 & _LVP_OFF

	errorlevel -207
	errorlevel -203
	errorlevel -205
	errorlevel -302

CBLOCK 0x20 ;max 80 bytes in bank 0

	ENDC

CBLOCK 0xA0 ;32 RAM Bytes

	ENDC

CBLOCK 0x70 ;16 bytes shared space
	RecievedByte
	Flags
	delay
	delay2
	
	FrameWork
	DigitSelect
	DigitRecieved
	
	Char1
	Char2
	Char3

	ResetCounter
	ResetCounterNest
	ENDC
	
;TLC5916 Defines
#define CLK LATA,1
#define OE LATC,4 
#define SDO LATA,0
#define LE LATA,2

;RA4, RA5, RC5 = Transistor switches
#define Digit1 LATC,5
#define Digit2 LATA,4
#define Digit3 LATA,5

;#define TXpin LATC,4
;#define RXpin LATC,5
;#define GenIO1 LATA,0 ; RC0 on test program, SRL
;#define GenIO2 LATA,1 ;RC2 on test program, CLK

#define SRLIn PORTA,1 ; RC0 on test program, SRL
#define CLKIn PORTA,0 ;RC2 on test program, CLK

#define cTMR2FULL b'00111110' ;1:8 post, 1:16 pre, TMRON, 6.55mS
#define cTMR2DIM  b'00001100' ;1:2 post, 1:1 pre, TMRON, DIM

#define cResetValue .2

#define		BANK0		movlb .0 ;should be bank 0

;Comment out to enable Slave Select on HEADER pin 4
#define SLAVESLECTENABLE

;V.3 NEW PINs:
;1 = V+
;2 = V-
;3 = RC1 - SPI DataIn
;4 = RC3 - SlaveSelect
;5 = RC0 - SPI CLK
;6 = RC2 - SPI DataOut
;7 = SW1
;8 = RA3/MCLR
;9 = unconnected/key
;10 = SW2

ORG 0x000	
	PAGESEL Initialize
	goto Initialize

ORG		0x004 ;INTERRUPT VECTOR	
	banksel SSP1BUF
	movfw SSP1BUF
	BANK0
	movwf RecievedByte
	
	BANK0
	movfw RecievedByte
	xorlw .251 ;if recieved is 251 = Dim On
	BTFSS STATUS, Z
	bra inta
	
	banksel T2CON
	movlw cTMR2DIM
	movwf T2CON
	BANK0	
	bra INTX
	
inta	
	movfw RecievedByte
	xorlw .250 ;if recieved is 250 = Dim OFF
	BTFSS STATUS, Z
	bra intc
	
	banksel T2CON
	movlw cTMR2FULL
	movwf T2CON
	bra INTX
	
intc	
	call FillCharBuffer	;fill a normal byte

	decf DigitRecieved,f

INTX
	movlw cResetValue
	movwf ResetCounter

	banksel PIR1
	BCF PIR1,SSP1IF  ;clear interrupt flag
	
		retfie ;returns and finishes orignal interrupt

;======================= PROGRAM START ====================================
Start
	banksel LATA
	BCF Digit1
	BCF Digit2
	BCF Digit3

;DP - D - C - E - A - F - B - G
	;initialize with all dashes displayed, user can alter to add custom 3 digit start up msg
	BANK0
	movlw .48 ;dash
	movwf Char1
	
	movlw .48 ;dash
	movwf Char2
	
	movlw .48 ;dash
	movwf Char3	

	;init variables
	movlw .3
	movwf DigitSelect ;starting digit

	movlw .2
	movwf DigitRecieved

	banksel LATC
	BCF OE ;Low=Output Drivers Enabled, High=Output Drivers Blanked
	banksel LATA
	BSF LE
	
;enter TLC5916 into normal mode.....	it works, not sure if 100% correct but works
	banksel LATA
	BCF CLK
	banksel LATC	
	BSF OE
	
	call NopDelay
	
	banksel LATC	
	BSF CLK
	
	call NopDelay
	banksel LATC	
	BCF CLK	
	
	banksel LATC
	BCF OE	
;end Normal Mode Configw

	BSF INTCON, GIE ;init Interrupts
	BSF INTCON, PEIE	
	BANK0
	
;Main Loop: Monitors for DrawFlag to push the next digit, drawflag is set by TMR2 Interrupt
	;As digits are pushed, a counter is ran and if new data isn't received before counter elapses
	;	it resets the Character Buffer variable to frame incoming data
Main
	banksel PIR1
	BTFSC PIR1, TMR2IF
	call Draw7Segment					
		bra Main ;start again
			
FillCharBuffer
	BANK0		
	movfw DigitRecieved
	BRW
	bra FillChar3
	bra FillChar2	
	bra FillChar1
	
FillChar1
	movfw RecievedByte
	movwf Char1
		return
	
FillChar2
	movfw RecievedByte
	movwf Char2
		return

FillChar3
	movfw RecievedByte
	movwf Char3
	
	movlw .3
	movwf DigitRecieved	
		return
		
Draw7Segment
	BCF PIR1, TMR2IF
	banksel TMR2
	clrf TMR2

	banksel LATA
	BSF LE
	banksel LATC	
	BSF OE
		
	decfsz DigitSelect,f
	goto T1
	
	movlw .3
	movwf DigitSelect
	
T1
	banksel LATC
	movfw DigitSelect
	brw ;table branch with DigitSelect
	nop	;no 0
	bra Ch3
	bra Ch2
	bra Ch1
		
Ch1
	banksel LATA
	BCF Digit2
	BCF Digit3
	banksel LATC	
	BSF Digit1	
	BANK0	
	
	movfw Char1
	call CharacterMapTable
	bra RunShift

Ch2
	banksel LATC	
	BCF Digit1
	banksel LATA	
	BCF Digit3
	BSF Digit2
	BANK0		
		
	movfw Char2
	call CharacterMapTable
	bra RunShift

Ch3
	banksel LATC	
	BCF Digit1
	banksel LATA	
	BCF Digit2
	BSF Digit3
	
	;counts number of digit draws, and wi	
	decfsz ResetCounter,f
	bra Ch3a

	movlw .2
	movwf DigitRecieved	
		
	movlw cResetValue
	movwf ResetCounter	
	
Ch3a	
	BANK0		
	movfw Char3
	call CharacterMapTable
	bra RunShift	
		

RunShift
;send data to TLC5916 with bit-banging
	movwf FrameWork

	banksel LATC
	BTFSC FrameWork,7 ;checks if the retrived data's bit 7
	BSF SDO ; is 1, if it is, SDO pin is set, and the shift register is 
	call ShiftRegister ; incremented, SDO is then returned to 0 state
					; by ShiftRegister call

	BTFSC FrameWork,6 ;repeats with all the bits
	BSF SDO
	call ShiftRegister

	BTFSC FrameWork,5
	BSF SDO
	call ShiftRegister

	BTFSC FrameWork,4
	BSF SDO
	call ShiftRegister

	BTFSC FrameWork,3
	BSF SDO
	call ShiftRegister

	BTFSC FrameWork,2
	BSF SDO
	call ShiftRegister

	BTFSC FrameWork,1
	BSF SDO
	call ShiftRegister

	BTFSC FrameWork,0
	BSF SDO
	call ShiftRegister

	banksel LATA
	BCF LE ; enable outputs
	banksel LATC	
	BCF OE
		return;goto Main

ShiftRegister ;increment the register
	banksel LATC	
	BSF CLK ; make CLK pin high
	call NopDelay
	call NopDelay		
	BCF CLK
	BCF SDO ;return SDO pin to Low
	call NopDelay
	call NopDelay		
		return

NopDelay ;delay routine
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	return

Initialize
	banksel PIE1
	clrf PIE1
	banksel IOCAP
	clrf IOCAP
	banksel IOCAN
	clrf IOCAN
	banksel CPSCON0
	clrf CPSCON0
	banksel T1GCON
	clrf T1GCON
	banksel CM1CON0
	clrf CM1CON0
	banksel CM1CON1
	clrf CM1CON1
	banksel CCP1CON
	clrf CCP1CON
	banksel CCP2CON
	clrf CCP2CON
	banksel CCP3CON
	clrf CCP3CON

 	banksel INTCON
       movlw   b'00000000' ;set peripherial interrupts, not GIE yet'
       movwf   INTCON
        
	banksel OSCCON
	movlw b'11110000' ;set for 8mhz
	movwf OSCCON
	
	; set up direction of I/O pins
	banksel TRISA
	movlw b'00000000'
	movwf TRISA ;all outputs

	;HEADER Pin 3 = RC1 - SPI DataIn - INPUT
	;HEADER Pin 4 = RC3 - SlaveSelect - INPUT
	;HEADER Pin 5 = RC0 - SPI CLK - INPUT
	;HEADER Pin 6 = RC2 - SPI DataOut - OUTPUT

	banksel TRISC	
	movlw b'00001011'
	movwf TRISC

	; set up A/D converter
	banksel ADCON1
	clrf ADCON1

	banksel ADCON0 ;good SDO, SS, CLK, SDI
	clrf  ADCON0

	banksel ANSELA
	clrf ANSELA
	
	banksel ANSELC
	CLRF ANSELC	

	banksel OPTION_REG
	movlw b'00000000' ;WPUx internal pullups enable
	movwf OPTION_REG

	banksel WPUA
	movlw b'00001000' ;enable MCLR weak pull ups
	movwf WPUA

	banksel WPUC
	movlw b'00001011'
	movwf WPUC
	
	banksel APFCON0 ;default pin settings
	clrf APFCON0
	banksel APFCON1
	clrf APFCON1

	;setup TMR2
	banksel T2CON
	movlw cTMR2FULL ;1:8 post, 1:16 pre, TMRON, 6.55mS
	movwf T2CON
	
;When initializing the SPI, several options need to be
;specified. This is done by programming the appropriate
;control bits (SSP1CON1<5:0> and SSP1STAT<7:6>).

;To enable the serial port, SSP1 Enable bit, SSP1EN of
;the SSP1CON1 register must be set. To reset or reconfigure
;SPI mode, clear the SSP1EN bit, re-initialize the
;SSP1CONx registers and then set the SSP1EN bit.

#ifdef SLAVESLECTENABLE ;Defined at top, commented out, Slave Select will be disabled
	movlw b'00100100' ;idle state low level, slave mode, SS enabled	
#else
	movlw b'00100101' ;idle state low level, slave mode, SS disabled
#endif

	banksel SSP1CON1
	movwf SSP1CON1 ;Load SPI configuration register

	banksel SSP1STAT
	movlw b'00000000' ;Input data sampled at end of data output time
	movwf SSP1STAT ;6: 0=Transmit occurs on transition from Idle to active clock state

	banksel SSP1BUF ;flush buffer
	movfw SSP1BUF

	banksel SSP1BUF ;flush buffer
	movfw SSP1BUF

	banksel PIE1
	BSF PIE1, SSP1IE ;set SPI interrupt enable
	
	movlw cResetValue ;set initial reset value
	movwf ResetCounter
		goto Start

;DP - D - C - E - A - F - B - G

CharacterMapTable ;character Map
brw
retlw b'01111110' ;0
retlw b'00100010' ;1
retlw b'01011011' ;2
retlw b'01101011' ;3
retlw b'00100111' ;4
retlw b'01101101' ;5
retlw b'01111101' ;6
retlw b'00101010' ;7
retlw b'01111111' ;8
retlw b'01101111' ;9
;LETTERS
retlw b'00111111' ;A - 10
retlw b'01110101' ;b
retlw b'01010001' ;c
retlw b'01110011' ;d
retlw b'01011101' ;E ;14
retlw b'00011101' ;F
retlw 0xFF ;g
retlw b'00110111' ;H
retlw b'00100000' ;i
retlw b'01100010' ;J
retlw 0xFF ;k
retlw b'01010100' ;L - 21
retlw 0xFF ;m
retlw b'00110001' ;n
retlw b'01110001' ;o
retlw b'00011111' ;P
retlw 0xFF ;q ;26
retlw b'00010001' ;r
retlw b'11101101' ;S.
retlw 0xFF ;t
retlw b'01110000' ;u
retlw 0xFF ;v
retlw 0xFF ;w
retlw 0xFF ;x
retlw b'01100111' ;y
retlw 0xFF ;z
retlw b'00000000' ;space ;36


retlw b'10000000' ;DOT, Decimal Place ; 37

retlw b'11111110' ;0 w/ DP
retlw b'10100010' ;1 w/ DP
retlw b'11011011' ;2 w/ DP ;40
retlw b'11101011' ;3 w/ DP
retlw b'10100111' ;4 w/ DP
retlw b'11101101' ;5 w/ DP
retlw b'11111101' ;6 w/ DP
retlw b'10101010' ;7 w/ DP
retlw b'11111111' ;8 w/ DP
retlw b'11101111' ;9 w/ DP ;47

retlw b'00000001' ; only middle dash 48
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder

retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder

retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder

retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder

retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder

retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder

retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder

retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder

retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder

retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder

retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder

retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder

retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder

retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder

retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder

retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder

retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder

retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder
retlw 0xFF ;placeholder

retlw 0x00 ;placeholder
retlw 0x00 ;placeholder
retlw 0x00 ;placeholder
retlw 0x00 ;placeholder
retlw 0x00 ;placeholder
retlw 0x00 ;placeholder
retlw 0x00 ;placeholder
retlw 0x00 ;placeholder
retlw 0x00 ;placeholder
retlw 0x00 ;placeholder

retlw 0x00 ;placeholder
retlw 0x00 ;placeholder
retlw 0x00 ;placeholder
retlw 0x00 ;placeholder
retlw 0x00 ;placeholder
retlw 0x00 ;placeholder
retlw 0x00 ;placeholder
retlw 0x00 ;placeholder
retlw 0x00 ;placeholder
retlw 0x00 ;placeholder

retlw 0x00 ;placeholder
retlw 0x00 ;placeholder
retlw 0x00 ;placeholder
retlw 0x00 ;placeholder
retlw 0x00 ;placeholder
retlw 0x00 ;placeholder
retlw 0x00 ;placeholder
retlw 0x00 ;placeholder
retlw 0x00 ;placeholder
retlw 0x00 ;placeholder

retlw 0x00 ;placeholder
retlw 0x00 ;placeholder
retlw 0x00 ;placeholder

		END	







